/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLUtil.java 98 2006-02-24 16:18:48Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import java.io.ByteArrayInputStream;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;

/**
 * Contains utilities methods for XML operations.
 * 
 * @version $Rev: 98 $ $Date: 2006-02-24 17:18:48 +0100 (ven., 24 f??vr. 2006) $
 * @since Petals 1.0
 * @author ddesjardins - eBMWebsourcing
 */
public final class XMLUtil {

    private XMLUtil() {
        // Do nothing
    }

    /**
     * Create a String result from a DOM document
     * 
     * @param document
     *            the DOM Document
     * @return a String representation of the DOM Document
     * @throws TransformerException
     */
    public static String createStringFromDOMDocument(Document document)
        throws TransformerException {
        document.normalize();
        Source source = new DOMSource(document);
        StringWriter out = new StringWriter();
        Result resultStream = new StreamResult(out);
        TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer;
        transformer = tFactory.newTransformer();
        transformer.transform(source, resultStream);
        String result = out.toString();

        return result;
    }

    /**
     * Create a document from a String
     * 
     * @param xml
     *            xml string
     * @return document representing the document
     */
    public static Document createDocumentFromString(String xml) {
        Document doc = null;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory
                .newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            byte[] msgByte = xml.getBytes();
            ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
            InputSource inputSource = new InputSource(in);
            doc = builder.parse(inputSource);
        } catch (Exception e) {
            // Do nothing
        }
        return doc;
    }

}
