/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentService.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.zip.ZipFile;

import javax.jbi.JBIException;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.AdminServiceMBean;
import javax.jbi.management.LifeCycleMBean;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.apache.commons.io.FileUtils;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.autoload.AutoLoaderImpl;
import org.objectweb.petals.jbi.management.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.LifeCycleAbstract;
import org.objectweb.petals.jbi.management.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.ServiceUnitLifeCycle;
import org.objectweb.petals.jbi.management.service.util.XMLResult;
import org.objectweb.petals.jbi.management.service.util.XMLResult.CauseFramework;
import org.objectweb.petals.jbi.management.service.util.XMLResult.MessageType;
import org.objectweb.petals.jbi.management.service.util.XMLResult.TaskResult;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.Connection;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceAssembly;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.ParameterCheckHelper;
import org.objectweb.petals.util.ZipUtil;
import org.objectweb.util.monolog.api.Logger;

import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_ADMIN_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.MONOLOG_FACTORY_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;

/**
 * JBI deployment service implementation. TODO map recovery service with fractal
 * 
 * @author Olivier Fabre - EBM Websourcing
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBMWebsourcing
 */
public class DeploymentService implements DeploymentServiceMBean,
    BindingController, LifeCycleController {

    protected static final String[] CLIENT_INTERFACES = new String[] {
        PLATFORM_REPOSITORY_NAME, JBI_LIFECYCLEMANAGER_NAME, JBI_ADMIN_NAME,
        JBI_SYSTEMSTATE_NAME, JBI_ENDPOINTSERVICE_NAME};

    protected EndpointService endpointService;

    /**
     * JBI Container :: Admin Service
     */
    protected AdminServiceMBean adminSrv;

    /**
     * Component lifecycle state.
     */
    protected String fcState;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog Logger instance.
     */
    protected Logger logger;

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected LifeCycleManagerService managerService;

    /**
     * Package Handler
     */
    protected PackageHandler packageHandler;

    /**
     * Loading packages cache.
     */
    protected List<String> packagesLoadedCache;

    /**
     * JBI Container :: Recovery Service
     */
    protected SystemState recoverySrv;

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositorySrv;

    /**
     * Default constructor.
     */
    public DeploymentService() {
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
     *      java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf)
        throws NoSuchInterfaceException, IllegalBindingException,
        IllegalLifeCycleException {
        if (LOGGER_NAME.equals(clientItfName)) {
            logger = (Logger) serverItf;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else if (PLATFORM_REPOSITORY_NAME.equals(clientItfName)) {
            repositorySrv = (RepositoryService) serverItf;
        } else if (JBI_LIFECYCLEMANAGER_NAME.equals(clientItfName)) {
            managerService = (LifeCycleManagerService) serverItf;
        } else if (JBI_ADMIN_NAME.equals(clientItfName)) {
            adminSrv = (AdminServiceMBean) serverItf;
        } else if (JBI_SYSTEMSTATE_NAME.equals(clientItfName)) {
            recoverySrv = (SystemState) serverItf;
        } else if (JBI_ENDPOINTSERVICE_NAME.equals(clientItfName)) {
            endpointService = (EndpointService) serverItf;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    // ----------------------------------------------------------------------
    // JBI DeploymentServiceMBean interface implementation
    // ----------------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#canDeployToComponent(java.lang.String)
     */
    public boolean canDeployToComponent(String componentName) {
        ParameterCheckHelper.isNullParameterWithLog(componentName,
            "componentName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
            "componentName", log);
        boolean result = false;
        ComponentLifeCycle clc = managerService
            .getComponentByName(componentName);
        result = clc != null
            && LifeCycleAbstract.STARTED.equals(clc.getCurrentState())
            && clc.getComponent().getServiceUnitManager() != null;
        return result;
    }

    /**
     * TODO check exception and xml string result
     */
    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#deploy(java.lang.String)
     */
    public String deploy(String serviceAssemblyZipUrl) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyZipUrl,
            "serviceAssemblyZipUrl", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyZipUrl,
            "serviceAssemblyZipUrl", log);
        try {
            new URL(serviceAssemblyZipUrl).toURI();
        } catch (Exception e) {
            String msg = "serviceAssemblyZipUrl is not correctly formed";
            log.error(msg);
            throw new IllegalArgumentException(msg);
        }
        log.start();
        String msg;
        String result = null;
        XMLResult xmlResult = new XMLResult("deploy");
        /*
         * Transforming URL into URI
         */
        URI archiveURI = packageHandler.processAndGetPackageURI(
            serviceAssemblyZipUrl, true);
        /*
         * Loading descriptor
         */
        JBIDescriptor descriptor = packageHandler.loadDescriptor(archiveURI);
        /*
         * Checking service assembly package
         */
        if (checkServiceAssembly(descriptor, archiveURI, xmlResult) == null) {
            /*
             * Exploding service assembly into the installation repository
             */
            String saName = getServiceAssemblyName(descriptor);
            URI installationRoot = packageHandler.expandPackageIntoRepository(
                saName, archiveURI);
            /*
             * Explode each service unit in the SA install root
             */
            ServiceAssembly saDesc = descriptor.getServiceAssembly();
            if (saDesc != null) {
                for (ServiceUnit suDesc : saDesc.getServiceUnits()) {
                    String suName = suDesc.getIdentification().getName();
                    String suZipNAme = suDesc.getTargetArtifactsZip();
                    URI suZipLocation = packageHandler.processAndGetPackageURI(
                        "file://" + installationRoot.getPath() + suZipNAme,
                        true);
                    expandSUIntoSA(suName, suZipLocation, saName);
                }
            }
            /*
             * Performing service assembly deployment
             */
            result = deploy(installationRoot, descriptor);
            /*
             * Creating state holder for service assembly
             */
            File file = getArchiveFile(archiveURI);
            File installedArchive = getInstalledArchive(file);
            recoverySrv.createServiceAssemblyStateHolder(saName,
                installationRoot.toURL().toString(), installedArchive.toURL()
                    .toString(), LifeCycleMBean.SHUTDOWN);
            /*
             * Copy archive file to installed dir
             */
            copyFile(file, installedArchive);
            /*
             * Create success file in workdir
             */
            createSuccessFile(installedArchive);
        } else {
            msg = "Service assembly package isn't validated";
            log.error(msg);
            throw new Exception(msg);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.jbi.management.service.DeploymentServiceMBean#deploy(java.net.URI,
     *      org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor)
     */
    public String deploy(URI installationRoot, JBIDescriptor descriptor)
        throws PetalsException {
        ParameterCheckHelper.isNullParameterWithLog(installationRoot,
            "installationRoot", log);
        ParameterCheckHelper.isNullParameterWithLog(descriptor, "descriptor",
            log);
        XMLResult xmlResult = new XMLResult("deploy");
        /*
         * Get the Service Assembly production element
         */
        ServiceAssembly saDesc = descriptor.getServiceAssembly();
        /*
         * Create a new ServiceAssemblyLifeCycle
         */
        ObjectName objectName = null;
        try {
            // Retrieve the DeploymentService ObjectName
            objectName = managerService.getDeploymentServiceMBeanName();
        } catch (ManagementException e) {
            return createErrorMessage(xmlResult,
                "Error while getting the object name from the MBean", null,
                false, null, null, new String[0]);
        }
        ServiceAssemblyLifeCycle saLifeCycle = createNewServiceAssemblyLifeCycle(
            objectName, saDesc, installationRoot);
        // TODO : verify that all Service units are present in the package and
        // that all targeted components are installed in the JBI System and
        // check duplicate service units as describe in spec section 6.6
        StringBuffer componentOutput = new StringBuffer();
        int nbFail = 0;

        /*
         * Deploy each sus to the targeted components.
         */
        String exceptionMessage = "Deployement of the service assembly {1} has failed. "
            + "The deployement of all the service units has failed.";
        List<String> exceptionParams = new ArrayList<String>();
        String saName = saDesc.getIdentification().getName();
        exceptionParams.add(saName);
        int posParams = 2;
        for (ServiceUnit suDesc : saDesc.getServiceUnits()) {
            // Get su name and root folder
            String suName = suDesc.getIdentification().getName();
            File saInstallRoot = repositorySrv.getComponentInstallRoot(saName);
            URI suRoot = (new File(saInstallRoot, suName)).toURI();
            // Deploy it to the targeted component
            String targetedComponent = suDesc.getTargetComponentName();
            ComponentLifeCycle clc = managerService
                .getComponentByName(targetedComponent);
            if (clc != null) {
                ServiceUnitManager sum = clc.getComponent()
                    .getServiceUnitManager();
                if (sum != null) {
                    String xmlReturn = null;
                    try {
                        xmlReturn = sum.deploy(suName, suRoot.getPath());
                    } catch (Exception e) {
                        return createErrorMessage(
                            xmlResult,
                            "Deployement of the service assembly {1} has failed. "+ e.getMessage(),
                            componentOutput, true, saName, suName,
                            new String[] {saName, suName});
                    }
                    if (!isValidComponentTask(xmlReturn, this.getClass()
                        .getClassLoader().getResourceAsStream(
                            "schema/management-message-component.xsd"))) {
                        xmlReturn = xmlResult.wrapComponentTaskResult(suDesc
                            .getTargetComponentName(), "deploy", xmlReturn);

                    }
                    if (xmlReturn.indexOf("SUCCESS") > -1) {
                        componentOutput.append(xmlReturn);
                        // Create and register a new ServiceUnitLifeCycle
                        ServiceUnitLifeCycle suLifeCycle = createNewServiceUnitLifeCycle(
                            suDesc, suRoot.getPath());
                        try {
                            saLifeCycle.registerSUIntoSA(suName, suLifeCycle);
                            // Update the result
                        } catch (JBIException e) {
                            // The service unit is duplicate
                            return createErrorMessage(
                                xmlResult,
                                "Deployement of the service assembly {1} has failed. Error while registering the service unit {2} life cycle.",
                                componentOutput, true, saName, suName,
                                new String[] {saName, suName});
                        }
                    } else {
                        nbFail++;
                        exceptionMessage = updateExceptionMessage(
                            exceptionParams, exceptionMessage, posParams,
                            saName, targetedComponent,
                            "The deployment of the service unit {" + posParams
                                + "} for the component", true);
                        posParams++;
                    }
                } else {
                    nbFail++;
                    exceptionMessage = updateExceptionMessage(
                        exceptionParams,
                        exceptionMessage,
                        posParams,
                        saName,
                        targetedComponent,
                        "The service unit manager is null for for the component {"
                            + posParams
                            + "}. You may need to initialise the component. No service unit can be deployed.",
                        false);
                    posParams++;
                }
            } else {
                nbFail++;
                exceptionMessage = updateExceptionMessage(
                    exceptionParams,
                    exceptionMessage,
                    posParams,
                    saName,
                    targetedComponent,
                    "The component {" + posParams
                        + "} is not deployed. No service unit can be deployed.",
                    false);
                posParams++;
            }
        }
        if (saDesc.getServiceUnits().size() != 0
            && nbFail >= saDesc.getServiceUnits().size()) {
            // The deployment of all the service unit has failed
            return createWarningMessage(xmlResult, exceptionMessage,
                exceptionParams, componentOutput);
        }
        /*
         * Register connections, if exist, into Registry
         */
        List<Connection> connections = saDesc.getConnections();
        if (connections != null) {
            for (Connection connection : connections) {
                String result = registerConnection(connection, xmlResult,
                    componentOutput, saName);
                if (result != null) {
                    return result;
                }
            }
        }
        /*
         * Register ServiceAssemblyLifeCycle into manager service map.
         */
        managerService.registerServiceAssembly(saName, saLifeCycle);
        /*
         * Update the result
         */
        xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
            "Successfully deployed service assembly : {1}",
            new String[] {saName}, CauseFramework.YES);
        xmlResult.addComponentTaskResult(componentOutput);
        /*
         * add service assembly to the loaded cache list
         */
        packagesLoadedCache.add(saName);
        String output = xmlResult.getString();
        try {
            XMLResult.validateManagementMessage(output, this.getClass()
                .getClassLoader().getResourceAsStream(
                    "schema/management-message.xsd"));
        } catch (Exception e1) {
            log.error("Problem while validating management message", e1);
        }
        return output;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getComponentsForDeployedServiceAssembly(java.lang.String)
     */
    public String[] getComponentsForDeployedServiceAssembly(
        String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        List<String> result = new ArrayList<String>();
        ServiceAssemblyLifeCycle saLifeCycle = managerService
            .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            List<ServiceUnit> sus = saLifeCycle.getServiceAssembly()
                .getServiceUnits();
            for (ServiceUnit unit : sus) {
                result.add(unit.getTargetComponentName());
            }
        } else {
            String msg = "Cannot retrieve service assembly with the specified name : "
                + serviceAssemblyName;
            log.error(msg);
            throw new Exception(msg);
        }
        return result.toArray(new String[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssemblies()
     */
    public String[] getDeployedServiceAssemblies() throws Exception {
        Set<String> sas = managerService.getServiceAssemblyLifeCycles()
            .keySet();
        return sas.toArray(new String[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssembliesForComponent(java.lang.String)
     */
    public String[] getDeployedServiceAssembliesForComponent(
        String componentName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(componentName,
            "componentName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
            "componentName", log);
        Set<String> result = new HashSet<String>();
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = managerService
            .getServiceAssemblyLifeCycles();
        for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
            for (ServiceUnit unit : sus) {
                if (componentName.equals(unit.getTargetComponentName())) {
                    result.add(salc.getServiceAssembly().getIdentification()
                        .getName());
                }
            }
        }
        return result.toArray(new String[0]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceUnitList(java.lang.String)
     */
    public String[] getDeployedServiceUnitList(String componentName)
        throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(componentName,
            "componentName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
            "componentName", log);
        Set<String> result = new HashSet<String>();
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = managerService
            .getServiceAssemblyLifeCycles();
        for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
            for (ServiceUnit unit : sus) {
                if (componentName.equals(unit.getTargetComponentName())) {
                    result.add(unit.getIdentification().getName());
                }
            }
        }
        return result.toArray(new String[0]);
    }

    // ----------------------------------------------------------------------
    // Fractal LifecycleController implementation
    // ----------------------------------------------------------------------
    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
     */
    public String getFcState() {
        return org.objectweb.fractal.api.control.LifeCycleController.STARTED
            .equals(fcState) ? org.objectweb.fractal.api.control.LifeCycleController.STARTED
            : org.objectweb.fractal.api.control.LifeCycleController.STOPPED;
    }

    public String getServiceAssemblyDescriptor(String serviceAssemblyName)
        throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        String result = "Service Assembly doesn't exist :"
            + serviceAssemblyName;
        ServiceAssemblyLifeCycle saLifeCycle = managerService
            .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            URI installationRoot = saLifeCycle.getInstallationRoot();
            // Load JBI descriptor as string
            URI jbiXmlUri = installationRoot
                .resolve(JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE.replace(
                    '\\', '/'));
            result = getJbiDescriptorAsString(jbiXmlUri);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#getState(java.lang.String)
     */
    public String getState(String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        String result = null;
        String msg;
        ServiceAssemblyLifeCycle saLifeCycle = managerService
            .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            result = saLifeCycle.getCurrentState();
        } else {
            msg = "Cannot retrieve service "
                + "assembly with the specified name : " + serviceAssemblyName;
            log.error(msg);
            throw new Exception(msg);
        }
        if (!(LifeCycleMBean.SHUTDOWN.equals(result)
            || LifeCycleMBean.STARTED.equals(result) || LifeCycleMBean.STOPPED
            .equals(result))) {
            msg = "Service Assembly state isn't correctly defined";
            log.error(msg);
            throw new Exception(msg);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#isDeployedServiceUnit(java.lang.String,
     *      java.lang.String)
     */
    public boolean isDeployedServiceUnit(String componentName,
        String serviceUnitName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceUnitName,
            "serviceUnitName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceUnitName,
            "serviceUnitName", log);
        ParameterCheckHelper.isNullParameterWithLog(componentName,
            "componentName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
            "componentName", log);
        boolean result = false;
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = managerService
            .getServiceAssemblyLifeCycles();
        for (Iterator iter = saLifeCycles.values().iterator(); iter.hasNext()
            && !result;) {
            ServiceAssemblyLifeCycle salc = (ServiceAssemblyLifeCycle) iter
                .next();
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
            for (Iterator iter2 = sus.iterator(); iter2.hasNext() && !result;) {
                ServiceUnit unit = (ServiceUnit) iter2.next();
                if (componentName.equals(unit.getTargetComponentName())
                    && serviceUnitName.equals(unit.getIdentification()
                        .getName())) {
                    result = true;
                }
            }
        }
        return result;
    }

    /**
     * Check if the given SA is already registered to the Petals platform
     * 
     * @param packageUUID
     * @return true if the given SA is already loaded, false otherwise
     */
    public boolean isLoaded(String saID) {
        return packagesLoadedCache.contains(saID);
    }

    // ----------------------------------------------------------------------
    // Fractal BindingController implementation
    // ----------------------------------------------------------------------
    /**
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return CLIENT_INTERFACES;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName)
        throws NoSuchInterfaceException {
        if (PLATFORM_REPOSITORY_NAME.equals(clientItfName)) {
            return repositorySrv;
        } else if (JBI_LIFECYCLEMANAGER_NAME.equals(clientItfName)) {
            return managerService;
        } else if (JBI_ADMIN_NAME.equals(clientItfName)) {
            return adminSrv;
        } else if (JBI_SYSTEMSTATE_NAME.equals(clientItfName)) {
            return recoverySrv;
        } else if (JBI_ENDPOINTSERVICE_NAME.equals(clientItfName)) {
            return endpointService;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#shutDown(java.lang.String)
     */
    public String shutDown(String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        XMLResult xmlResult = new XMLResult("shutdown");
        ServiceAssemblyLifeCycle saLifeCycle = managerService
            .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            try {
                saLifeCycle.shutDown();
            } catch (Exception e) {
                if (e.getCause() == null) {
                    xmlResult
                        .addFrameworkTaskResult(
                            TaskResult.FAILED,
                            MessageType.ERROR,
                            null,
                            null,
                            "1",
                            "The service assembly {1} cannot be shutdowned in this state : {2}",
                            new String[] {serviceAssemblyName,
                                saLifeCycle.getCurrentState()},
                            CauseFramework.YES);
                } else {
                    StringWriter sw = new StringWriter();
                    PrintWriter pw = new PrintWriter(sw);
                    e.printStackTrace(pw);
                    xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
                        MessageType.ERROR, null, null, "1",
                        "The service assembly {1} cannot be shutdowned.",
                        new String[] {serviceAssemblyName}, sw.toString(),
                        CauseFramework.YES);
                }
                String output = xmlResult.getString();
                try {
                    XMLResult.validateManagementMessage(output, this.getClass()
                        .getClassLoader().getResourceAsStream(
                            "schema/management-message.xsd"));
                } catch (Exception e1) {
                    log
                        .error("Problem while validating management message",
                            e1);
                }
                return output;
            }
        } else {
            xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
                MessageType.ERROR, null, null, "1",
                "The service assembly {1} is not deployed",
                new String[] {serviceAssemblyName}, CauseFramework.YES);
            String output = xmlResult.getString();
            try {
                XMLResult.validateManagementMessage(output, this.getClass()
                    .getClassLoader().getResourceAsStream(
                        "schema/management-message.xsd"));
            } catch (Exception e1) {
                log.error("Problem while validating management message", e1);
            }
            return output;
        }
        xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
            "Successfully shutdown service assembly : {1}",
            new String[] {serviceAssemblyName}, CauseFramework.YES);
        String output = xmlResult.getString();
        try {
            XMLResult.validateManagementMessage(output, this.getClass()
                .getClassLoader().getResourceAsStream(
                    "schema/management-message.xsd"));
        } catch (Exception e1) {
            log.error("Problem while validating management message", e1);
        }
        return output;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#start(java.lang.String)
     */
    public String start(String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        XMLResult xmlResult = new XMLResult("start");
        ServiceAssemblyLifeCycle saLifeCycle = managerService
            .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            try {
                saLifeCycle.start();
            } catch (Exception e) {
                if (e.getCause() == null) {
                    xmlResult
                        .addFrameworkTaskResult(
                            TaskResult.FAILED,
                            MessageType.ERROR,
                            null,
                            null,
                            "1",
                            "The service assembly {1} cannot be started in this state : {2}",
                            new String[] {serviceAssemblyName,
                                saLifeCycle.getCurrentState()},
                            CauseFramework.YES);
                } else {
                    StringWriter sw = new StringWriter();
                    PrintWriter pw = new PrintWriter(sw);
                    e.printStackTrace(pw);
                    xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
                        MessageType.ERROR, null, null, "1",
                        "The service assembly {1} cannot be started.",
                        new String[] {serviceAssemblyName}, sw.toString(),
                        CauseFramework.YES);
                }
                String output = xmlResult.getString();
                try {
                    XMLResult.validateManagementMessage(output, this.getClass()
                        .getClassLoader().getResourceAsStream(
                            "schema/management-message.xsd"));
                } catch (Exception e1) {
                    log
                        .error("Problem while validating management message",
                            e1);
                }
                return output;
            }
        } else {
            xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
                MessageType.ERROR, null, null, "1",
                "The service assembly {1} is not deployed",
                new String[] {serviceAssemblyName}, CauseFramework.YES);
            String output = xmlResult.getString();
            try {
                XMLResult.validateManagementMessage(output, this.getClass()
                    .getClassLoader().getResourceAsStream(
                        "schema/management-message.xsd"));
            } catch (Exception e1) {
                log.error("Problem while validating management message", e1);
            }
            return output;
        }
        xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
            "Successfully started service assembly : {1}",
            new String[] {serviceAssemblyName}, CauseFramework.YES);
        String output = xmlResult.getString();
        try {
            XMLResult.validateManagementMessage(output, this.getClass()
                .getClassLoader().getResourceAsStream(
                    "schema/management-message.xsd"));
        } catch (Exception e1) {
            log.error("Problem while validating management message", e1);
        }
        return output;
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    public void startFc() throws IllegalLifeCycleException {
        init();
        fcState = org.objectweb.fractal.api.control.LifeCycleController.STARTED;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#stop(java.lang.String)
     */
    public String stop(String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
            "serviceAssemblyName", log);
        XMLResult xmlResult = new XMLResult("stop");
        String output = null;
        ServiceAssemblyLifeCycle saLifeCycle = managerService
            .getServiceAssemblyByName(serviceAssemblyName);
        if (saLifeCycle != null) {
            try {
                saLifeCycle.stop();
            } catch (Exception e) {
                if (e.getCause() == null) {
                    xmlResult
                        .addFrameworkTaskResult(
                            TaskResult.FAILED,
                            MessageType.ERROR,
                            null,
                            null,
                            "1",
                            "The service assembly {1} cannot be stopped in this state : {2}",
                            new String[] {serviceAssemblyName,
                                saLifeCycle.getCurrentState()},
                            CauseFramework.YES);
                } else {
                    StringWriter sw = new StringWriter();
                    PrintWriter pw = new PrintWriter(sw);
                    e.printStackTrace(pw);
                    xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
                        MessageType.ERROR, null, null, "1",
                        "The service assembly {1} cannot be stopped.",
                        new String[] {serviceAssemblyName}, sw.toString(),
                        CauseFramework.YES);
                }
                output = xmlResult.getString();
                try {
                    XMLResult.validateManagementMessage(output, this.getClass()
                        .getClassLoader().getResourceAsStream(
                            "schema/management-message.xsd"));
                } catch (Exception e1) {
                    log
                        .error("Problem while validating management message",
                            e1);
                }
                return output;
            }
        } else {
            xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
                MessageType.ERROR, null, null, "1",
                "The service assembly {1} is not deployed",
                new String[] {serviceAssemblyName}, CauseFramework.YES);
            output = xmlResult.getString();
            try {
                XMLResult.validateManagementMessage(output, this.getClass()
                    .getClassLoader().getResourceAsStream(
                        "schema/management-message.xsd"));
            } catch (Exception e1) {
                log.error("Problem while validating management message", e1);
            }
            return output;
        }
        xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
            "Successfully stopped service assembly : {1}",
            new String[] {serviceAssemblyName}, CauseFramework.YES);
        output = xmlResult.getString();
        try {
            XMLResult.validateManagementMessage(output, this.getClass()
                .getClassLoader().getResourceAsStream(
                    "schema/management-message.xsd"));
        } catch (Exception e1) {
            log.error("Problem while validating management message", e1);
        }
        return output;
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
     */
    public void stopFc() throws IllegalLifeCycleException {
        shutdown();
        fcState = org.objectweb.fractal.api.control.LifeCycleController.STOPPED;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        if (LOGGER_NAME.equals(clientItfName)) {
            logger = null;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else if (PLATFORM_REPOSITORY_NAME.equals(clientItfName)) {
            repositorySrv = null;
        } else if (JBI_LIFECYCLEMANAGER_NAME.equals(clientItfName)) {
            managerService = null;
        } else if (JBI_ADMIN_NAME.equals(clientItfName)) {
            adminSrv = null;
        } else if (JBI_SYSTEMSTATE_NAME.equals(clientItfName)) {
            recoverySrv = null;
        } else if (JBI_ENDPOINTSERVICE_NAME.equals(clientItfName)) {
            endpointService = null;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.management.DeploymentServiceMBean#undeploy(java.lang.String)
     */
    public String undeploy(String serviceAssemblyName) throws Exception {
        ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
            "serviceAssemblyZipUrl", log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
            "serviceAssemblyZipUrl", log);
        XMLResult xmlResult = new XMLResult("undeploy");
        /*
         * Retrieve service assembly lifecycle
         */
        ServiceAssemblyLifeCycle saLifeCycle = managerService
            .getServiceAssemblyByName(serviceAssemblyName);
        StringBuffer componentOuput = new StringBuffer();
        if (saLifeCycle != null) {
            /*
             * Put the service assembly in the shutdown state
             */
            String saState = saLifeCycle.getCurrentState();
            if (LifeCycleAbstract.STARTED.equals(saState)) {
                saLifeCycle.stop();
                saLifeCycle.shutDown();
            } else if (LifeCycleAbstract.STOPPED.equals(saState)) {
                saLifeCycle.shutDown();
            }
            saState = saLifeCycle.getCurrentState();
            if (saState.equals(LifeCycleAbstract.SHUTDOWN)) {
                /*
                 * The service assembly is in the SHUTDOWN state We can try to
                 * undeploy it
                 */
                String exceptionMessage = "Undeployement of the service assembly {1} has failed. "
                    + "The deployement of all the service units has failed.";
                List<String> exceptionParams = new ArrayList<String>();
                exceptionParams.add(serviceAssemblyName);
                int posParams = 2;
                int nbFail = 0;

                /*
                 * Jbi spec specifies that service units must be undeployed in
                 * the reverse order of the declared list in service assembly
                 * descriptor
                 */
                List<ServiceUnit> suList = saLifeCycle.getServiceAssembly()
                    .getServiceUnits();
                Collections.reverse(suList);
                for (ServiceUnit suDesc : suList) {
                    /*
                     * Undeploy it
                     */
                    String suName = suDesc.getIdentification().getName();
                    ServiceUnitLifeCycle serviceUnitLifeCycle = saLifeCycle
                        .getServiceUnitsLifeCycles().get(suName);
                    String targetedComponent = suDesc.getTargetComponentName();
                    ComponentLifeCycle clc = managerService
                        .getComponentByName(targetedComponent);
                    if (clc != null) {
                        ServiceUnitManager sum = clc.getComponent()
                            .getServiceUnitManager();
                        if (sum != null) {
                            String xmlReturn = null;
                            try {
                                xmlReturn = sum.undeploy(suName,
                                    serviceUnitLifeCycle
                                        .getServiceUnitRootPath());
                            } catch (Exception e) {
                                return createErrorMessage(
                                    xmlResult,
                                    "Undeployement of the service assembly {1} has failed. The shared library {2} is not installed.",
                                    componentOuput, false, serviceAssemblyName,
                                    suName, new String[] {serviceAssemblyName,
                                        "XMLManipulation"});
                            }
                            if (!isValidComponentTask(xmlReturn, this
                                .getClass().getClassLoader()
                                .getResourceAsStream(
                                    "schema/management-message-component.xsd"))) {
                                /*
                                 * The undeploy message does not complies to th
                                 * schema, it is wrapped in a valid one
                                 */
                                xmlReturn = xmlResult.wrapComponentTaskResult(
                                    suDesc.getTargetComponentName(),
                                    "undeploy", xmlReturn);
                            }
                            if (xmlReturn.indexOf("SUCCESS") > -1) {
                                componentOuput.append(xmlReturn);
                                /*
                                 * Remove the service unit from the service
                                 * assembly
                                 */
                                saLifeCycle.getServiceUnitsLifeCycles().remove(
                                    suName);
                            } else {
                                nbFail++;
                                exceptionMessage += "The undeployment of the service unit {"
                                    + posParams + "} for the component";
                                posParams++;
                                exceptionMessage += " {" + posParams
                                    + "} has failed.";
                                posParams++;
                                exceptionParams.add(suName);
                                exceptionParams.add(targetedComponent);
                            }
                        } else {
                            nbFail++;
                            exceptionMessage += "The service unit manager is null for for the component {"
                                + posParams
                                + "}. You may need to initialise the component. The service units cannot be undeployed.";
                            posParams++;
                            exceptionParams.add(targetedComponent);
                        }
                    } else {
                        exceptionMessage += "The component {"
                            + posParams
                            + "} is not deployed. The service units cannot be undeployed.";
                        posParams++;
                        exceptionParams.add(targetedComponent);
                    }
                }
                if (saLifeCycle.getServiceAssembly().getServiceUnits().size() != 0
                    && nbFail >= saLifeCycle.getServiceAssembly()
                        .getServiceUnits().size()) {
                    /*
                     * The deployement of all the service unit has failed
                     */
                    xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
                        MessageType.WARNING, null, null, "1", exceptionMessage,
                        exceptionParams.toArray(new String[0]),
                        CauseFramework.YES);
                    xmlResult.addComponentTaskResult(componentOuput);
                    String output = xmlResult.getString();
                    try {
                        XMLResult.validateManagementMessage(output, this
                            .getClass().getClassLoader().getResourceAsStream(
                                "schema/management-message.xsd"));
                    } catch (Exception e1) {
                        log.error(
                            "Problem while validating management message", e1);
                    }
                    return output;
                }
            } else {
                xmlResult
                    .addFrameworkTaskResult(
                        TaskResult.FAILED,
                        MessageType.ERROR,
                        null,
                        null,
                        "1",
                        "The service assembly {1} is not in the SHUTDOWN state. Current state : {2}",
                        new String[] {serviceAssemblyName,
                            saLifeCycle.getCurrentState()}, CauseFramework.YES);
                String output = xmlResult.getString();
                try {
                    XMLResult.validateManagementMessage(output, this.getClass()
                        .getClassLoader().getResourceAsStream(
                            "schema/management-message.xsd"));
                } catch (Exception e1) {
                    log
                        .error("Problem while validating management message",
                            e1);
                }
                return output;
            }
        } else {
            xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
                MessageType.ERROR, null, null, "1",
                "The service assembly {1} is not deployed",
                new String[] {serviceAssemblyName}, CauseFramework.YES);
            String output = xmlResult.getString();
            try {
                XMLResult.validateManagementMessage(output, this.getClass()
                    .getClassLoader().getResourceAsStream(
                        "schema/management-message.xsd"));
            } catch (Exception e1) {
                log.error("Problem while validating management message", e1);
            }
            return output;
        }
        /*
         * Remove possible connections
         */
        ServiceAssembly saDesc = saLifeCycle.getServiceAssembly();
        List<Connection> connections = saDesc.getConnections();
        if (connections != null) {
            for (Connection connection : connections) {
                QName consInterface = connection.getConsumerInterfaceName();
                QName consService = connection.getConsumerServiceName();
                String consEndpoint = connection.getConsumerEndpointName();
                QName provService = connection.getProviderServiceName();
                String provEndpoint = connection.getProviderEndpointName();

                if (provEndpoint != null && provService != null) {
                    if (consInterface != null) {
                        endpointService.deleteConnection(consInterface,
                            provService, provEndpoint);
                    } else if (provService != null && provEndpoint != null) {
                        endpointService.deleteConnection(consService,
                            consEndpoint, provService, provEndpoint);
                    }
                }
            }
        }

        /*
         * Remove the Service assembly folder.
         */
        saLifeCycle.removePackage();
        /*
         * Remove the service assembly life cycle from the manager service SA
         * Map
         */
        managerService.unregisterServiceAssembly(serviceAssemblyName);

        /*
         * Remove service assembly from state holder file
         */
        recoverySrv.deleteServiceAssemblyStateHolder(serviceAssemblyName);

        /*
         * Clean package loaded
         */
        packagesLoadedCache.remove(serviceAssemblyName);

        /*
         * Update the result
         */
        xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
            "Successfully deployed service assembly : {1}",
            new String[] {serviceAssemblyName}, CauseFramework.YES);
        xmlResult.addComponentTaskResult(componentOuput);
        String output = xmlResult.getString();
        try {
            XMLResult.validateManagementMessage(output, this.getClass()
                .getClassLoader().getResourceAsStream(
                    "schema/management-message.xsd"));
        } catch (Exception e1) {
            log.error("Problem while validating management message", e1);
        }
        return output;
    }

    /**
     * Check the validity of the service assembly archive
     * 
     * @param descriptor
     *            the sa JBIDescriptor
     * @param archiveURI
     *            URI representing the archive location 6.9 of the JBI Spec)
     * @return null if checking is completly ok and an xml String otherwise
     */
    protected String checkServiceAssembly(JBIDescriptor descriptor,
        URI archiveURI, XMLResult xmlResult) throws PetalsException {
        String result = null;
        String msg = null;

        /*
         * check if jbi descriptor has a service assembly element
         */

        ServiceAssembly sa = descriptor.getServiceAssembly();
        if (sa == null) {
            msg = "The service assembly production element was not found in "
                + "JBI Descriptor within deployment package: "
                + archiveURI.getPath();
            log.error(msg);
            // TODO return the real XMLManagementMessage
            xmlResult
                .addFrameworkTaskResult(
                    TaskResult.FAILED,
                    MessageType.ERROR,
                    null,
                    null,
                    "1",
                    "Other installation package already exists loaded in the service assembly framework: {1}.",
                    new String[] {"saName"}, CauseFramework.YES);
            result = xmlResult.getString();
        } else {

            /*
             * check if it is already loaded by package uuid
             */

            String saName = sa.getIdentification().getName();
            if (isLoaded(saName)) {
                // log error and return failure
                msg = "Other deployement package already exists loaded in "
                    + "the service assembly framework: " + saName;
                log.error(msg);
                xmlResult
                    .addFrameworkTaskResult(
                        TaskResult.FAILED,
                        MessageType.ERROR,
                        null,
                        null,
                        "1",
                        "Other installation package already exists loaded in the service assembly framework: {1}.",
                        new String[] {"saName"}, CauseFramework.YES);
                result = xmlResult.getString();
            } else {

                /*
                 * Check that all declared su are presents in the sa package
                 */

                List<ServiceUnit> serviceUnits = sa.getServiceUnits();

                ZipFile zipArchive = openZipFile(archiveURI);

                try {
                    for (ServiceUnit su : serviceUnits) {

                        String artifactZip = su.getTargetArtifactsZip();

                        if (!hasEntry(zipArchive, artifactZip)) {
                            /*
                             * specified target artifact zip not found within
                             * archive
                             */
                            msg = "The specified target artifact zip in JBI "
                                + "Descriptor's service unit declaration was not "
                                + "found within deployment package Zip archive file.";
                            throw new PetalsException(msg);
                        }
                    }
                } finally {
                    try {
                        zipArchive.close();
                    } catch (IOException e) {
                        throw new PetalsException(
                            "Error while closing the Zip archive file.", e);
                    }
                }
            }

        }
        return result;
    }

    /**
     * Copy a file
     * 
     * @param src
     *            source file
     * @param dst
     *            destination file
     * @throws IOException
     *             if an error occurs during copy process
     */
    protected void copyFile(File src, File dst) throws IOException {
        FileUtils.copyFile(src, dst);
    }

    /**
     * Create an error message
     * 
     * @param xmlResult
     * @param reason
     *            the error reason
     * @param componentOuput
     * @param addComponentTaskResult
     * @param saName
     *            service assembly name
     * @param suName
     *            service unit name
     * @param stringArray
     * @return a valid XML string conforming to the deployment task schema in
     *         JBI specification
     */
    protected String createErrorMessage(XMLResult xmlResult, String reason,
        StringBuffer componentOutput, boolean addComponentTaskResult,
        String saName, String suName, String[] stringArray) {
        if (addComponentTaskResult) {
            xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
                MessageType.ERROR, null, null, "1", reason, stringArray,
                CauseFramework.YES);
            xmlResult.addComponentTaskResult(componentOutput);
        } else {
            xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
                MessageType.ERROR, null, null, "1", reason, stringArray, "...",
                CauseFramework.YES);
        }
        String output = xmlResult.getString();
        try {
            XMLResult.validateManagementMessage(output, this.getClass()
                .getClassLoader().getResourceAsStream(
                    "schema/management-message.xsd"));
        } catch (Exception e1) {
            log.error("Problem while validating management message", e1);
        }
        return output;

    }

    /**
     * Creates a new ServciceAssemblyLifecycle
     * 
     * @param objectName
     * @param saDesc
     * @param installationRoot
     * @return a new ServiceAssemblyLifeCycle
     */
    protected ServiceAssemblyLifeCycle createNewServiceAssemblyLifeCycle(
        ObjectName objectName, ServiceAssembly saDesc, URI installationRoot) {
        return new ServiceAssemblyLifeCycle(objectName, log, saDesc,
            repositorySrv, recoverySrv, installationRoot);
    }

    /**
     * Creates a new ServiceUnitLifeCycle
     * 
     * @param suDesc
     * @param rootPath
     * @return a new ServiceUnitLifeCycle instance
     */
    protected ServiceUnitLifeCycle createNewServiceUnitLifeCycle(
        ServiceUnit suDesc, String rootPath) {
        return new ServiceUnitLifeCycle(suDesc, managerService, rootPath, log);
    }

    // ----------------------------------------------------------------------
    // Deployment processing methods
    // ----------------------------------------------------------------------

    protected boolean createSuccessFile(File installedArchive)
        throws IOException {
        return new File(AutoLoaderImpl.getWorkDirectory(), installedArchive
            .getName()
            + ".success").createNewFile();
    }

    // ----------------------------------------------------------------------
    // utility methods
    // ----------------------------------------------------------------------

    /**
     * Create a Warning message
     * 
     * @param xmlResult
     * @param exceptionMessage
     * @param exceptionParams
     * @param componentOuput
     * @return
     */
    protected String createWarningMessage(XMLResult xmlResult,
        String exceptionMessage, List<String> exceptionParams,
        StringBuffer componentOuput) {
        xmlResult.addFrameworkTaskResult(TaskResult.FAILED,
            MessageType.WARNING, null, null, "1", exceptionMessage,
            exceptionParams.toArray(new String[0]), CauseFramework.YES);
        xmlResult.addComponentTaskResult(componentOuput);
        String output = xmlResult.getString();
        try {
            XMLResult.validateManagementMessage(output, this.getClass()
                .getClassLoader().getResourceAsStream(
                    "schema/management-message.xsd"));
        } catch (Exception e1) {
            log.error("Problem while validating management message", e1);
        }
        return output;
    }

    /**
     * Expand a service unit package into its service assembly installRoot.
     * 
     * @param suId
     *            the unique identifier of the su
     * @param suZipLocation
     *            URI representing the su zip location
     * @param saId
     *            the unique identifier of the sa
     * @return SU installationRoot, null if it fails
     */
    protected URI expandSUIntoSA(String suId, URI suZipLocation, String saId) {
        log.start();
        String msg;
        URI result = null;
        /*
         * explode the installation package Zip archive into platform repository
         */
        try {
            result = repositorySrv.explodeSUIntoSAInstallFolder(suId,
                suZipLocation, saId);
        } catch (PetalsException pe) {
            msg = "Unexpected error adding shared library JBI installation "
                + "package into repository. " + pe.getMessage();
            log.error(msg, pe);
        }
        if (result == null) {
            msg = "Unexpected Platform repository service error. The "
                + "su root must NOT be null: " + suId;
            log.error(msg);
        }
        log.end();
        return result;
    }

    // ----------------------------------------------------------------------
    // Checking methods
    // ----------------------------------------------------------------------

    /**
     * Get the file according to the archive URI
     * 
     * @param archiveURI
     *            URI of the archive
     * @return a File pointing on the archive
     */
    protected File getArchiveFile(URI archiveURI) {
        return new File(archiveURI);
    }

    /**
     * Get a file of the given name in the component install directory
     * 
     * @param file
     *            the file to look for
     * @return a File pointing on the file name in the component installed
     *         directory
     */
    protected File getInstalledArchive(File file) {
        return new File(AutoLoaderImpl.getInstalledDirectory(), file.getName());
    }

    /**
     * Return a string representation of the JBI descriptor. This descriptor in
     * found thanks to the jbiXmlUri
     * 
     * @param jbiXmlUri
     *            the URI of the deployment descriptor
     * @return a string representing the JBI deployment descriptor
     */
    protected String getJbiDescriptorAsString(URI jbiXmlUri) {
        JBIDescriptorBuilder builder = new JBIDescriptorBuilder(jbiXmlUri,
            (java.util.logging.Logger) logger);
        return builder.getJbiDescriptorAsString();
    }

    /**
     * Retrieve the service assembly name found in the JBI deployment descriptor
     * 
     * @param descriptor
     *            the deployment descriptor
     * @return the service assembly name
     */
    protected String getServiceAssemblyName(JBIDescriptor descriptor) {
        return descriptor.getServiceAssembly().getIdentification().getName();
    }

    /**
     * Test if a zip file has a specific entry
     * 
     * @param archive
     *            zip file
     * @param entryName
     *            entry name
     * @return true if the entry is in the zip file
     */
    protected boolean hasEntry(ZipFile zipArchive, String artifactZip) {
        return ZipUtil.hasEntry(zipArchive, artifactZip);
    }

    protected void init() {
        log = new LoggingUtil(logger);
        log.start();

        /*
         * initialize packages loading cache
         */
        packagesLoadedCache = new ArrayList<String>();

        /*
         * Initialize package handler
         */
        packageHandler = new PackageHandler(logger, repositorySrv);

        log.end();
    }

    /**
     * Check if a component task result is valid against a xml schema
     * 
     * @param componentTask
     *            xml of the component task
     * @return true if valid, false if not valid
     */
    protected boolean isValidComponentTask(String componentTask, InputStream xsd) {
        return XMLResult.isValidComponentTask(componentTask, xsd);
    }

    /**
     * Load a zip file from a URI
     * 
     * @param archiveURI
     *            uri of the zip file
     * @return zip file
     * @throws PetalsException
     *             when problems
     */
    protected ZipFile openZipFile(URI archiveURI) throws PetalsException {
        return ZipUtil.openZipFile(archiveURI);
    }

    /**
     * Register a connection in the JBI container
     * 
     * @param connection
     *            The connection to register
     * @param xmlResult
     * @param componentOuput
     * @param saName
     *            Service assembly name
     * @return null if connection was succesfully registered
     */
    protected String registerConnection(Connection connection,
        XMLResult xmlResult, StringBuffer componentOuput, String saName) {
        String result = null;
        QName consInterface = connection.getConsumerInterfaceName();
        QName consService = connection.getConsumerServiceName();
        String consEndpoint = connection.getConsumerEndpointName();
        QName provService = connection.getProviderServiceName();
        String provEndpoint = connection.getProviderEndpointName();

        if (provEndpoint != null && provService != null) {
            try {
                if (consInterface != null) {

                    endpointService.createConnection(consInterface,
                        provService, provEndpoint);

                } else {
                    endpointService.createConnection(consService, consEndpoint,
                        provService, provEndpoint);
                }
            } catch (JBIException e) {
                // connection can't be correctly added
                result = createErrorMessage(
                    xmlResult,
                    "Deployement of the service assembly {1} has failed. Error while registering the service unit {2} connection.",
                    componentOuput, true, saName, null, new String[] {saName});
            }
        } else {
            result = createErrorMessage(
                xmlResult,
                "Deployement of the service assembly {1} has failed. Error while registering the service unit {2} connection.",
                componentOuput, true, saName, null, new String[] {saName});
        }
        return result;
    }

    protected void shutdown() {
        log.start();

        logger = null;

        packagesLoadedCache = null;

        log.end();
    }

    /**
     * Update the exception message by adding message to it
     * 
     * @param exceptionParams
     * @param exceptionMessage
     *            message to be updated
     * @param posParams
     *            parmater position number
     * @param suName
     *            service unit name
     * @param targeted
     *            Component component name
     * @param message
     *            message to be added
     * @param addSuName
     * @return the modifyed exception message
     */
    protected String updateExceptionMessage(List<String> exceptionParams,
        String exceptionMessage, int posParams, String suName,
        String targetedComponent, String message, boolean addSuName) {
        exceptionMessage += message;
        posParams++;
        exceptionMessage += " {" + posParams + "} is not valid.";
        posParams++;
        if (addSuName) {
            exceptionParams.add(suName);
        }
        exceptionParams.add(targetedComponent);
        return exceptionMessage;
    }

}
