/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationServiceMBean.java 154 2006-03-27 15:30:10Z alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.jbi.management.service;

import java.net.URI;

import javax.management.ObjectName;

import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;

/**
 * The <code>InstallationService</code> mbean implementation has to have its
 * mbean interface in the same package as it. This custom interface add methods
 * used by reinstallation tasks of the SystemState service.
 * 
 * @author alouis
 * 
 */
public interface InstallationServiceMBean extends
        javax.jbi.management.InstallationServiceMBean {

    /**
     * Install a shared library to the Petals platform. Installation steps :
     * create the sl classloader, register sl to the loaded sl list
     * 
     * @param installationRoot
     *            <code>URI</code> representing the install folder of the
     *            shared lib, must be non-null
     * @param descriptor
     *            the shared lib <code>JBIDescriptor</code>, must be non-null
     * @return return the slName, null in error cases
     */
    String installSharedLibrary(URI installationRoot, JBIDescriptor descriptor);

    /**
     * Install a component to the Petals platform. Installation steps : create
     * component and installation contexts, create and register a new
     * InstallerMBean for the component, register component to the loaded
     * component list
     * 
     * @param installationRoot
     *            <code>URI</code> representing the install folder of the
     *            component
     * @param descriptor
     *            the comp <code>JBIDescriptor</code>
     * @return the new InstallerMBean name, null in error cases
     */
    ObjectName loadNewInstaller(URI installationRoot, JBIDescriptor descriptor);
}
