/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2006 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: MessageSerializer.java,v 0.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.transport;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.net.URI;
import java.util.Enumeration;
import java.util.Iterator;

import javax.jbi.JBIException;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jms.Message;
import javax.jms.QueueSession;
import javax.jms.TextMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.messaging.NormalizedMessageImpl;
import org.objectweb.petals.util.NamingHelper;

/**
 * FIXME not finished. The Transporter uses the ObjectSerializer instead.
 * 
 * @version $Rev: 477 $ $Date: 2006-05-29 17:18:07 +0200 (lun., 29 mai 2006) $
 * @since Petals 1.0
 * @author alouis
 */
public class MessageSerializer implements Serializer {

    public static final String CONSUMER = "consumer";

    public static final String ENDPOINT = "endpoint-name";

    public static final String ENDPOINT_SERVICE = "endpoint-servicename";

    public static final String EXCHANGE_PROPERTIES_PREFIX = "exchProp_";

    public static final String ID = "id";

    public static final String INTERFACE = "interface";

    public static final String MESSAGE_TYPE = "type";

    public static final String MESSAGE_TYPE_FAULT = "fault";

    public static final String MESSAGE_TYPE_IN = "in";

    public static final String MESSAGE_TYPE_OUT = "out";

    public static final String NORMALIZED_MSG_PROPERTIES_PREFIX = "nrmlizeProp_";

    public static final String OPERATION = "operation";

    public static final String PATTERN = "pattern";

    public static final String PROVIDER = "provider";

    public static final String ROLE = "role";

    public static final String SERVICE = "service";

    public static final String STATUS = "status";

    public static final String TERMINATED = "terminated";

    public static final String TRANSACTED = "transacted";

    public Message jbi2jms(MessageExchangeImpl jbi, QueueSession queueSession)
        throws JBIException {
        // create text message
        TextMessage jms;
        try {
            jms = queueSession.createTextMessage();

            // //////////////////////////////////////////////////
            // set properties defined by the specification
            // //////////////////////////////////////////////////

            // set Id
            jms.setStringProperty(ID, jbi.getExchangeId());

            // set Service
            if (jbi.getService() != null) {
                String serviceName = NamingHelper.qNameToString(jbi
                    .getService());
                jms.setStringProperty(SERVICE, serviceName == null ? ""
                    : serviceName);
            }

            // set operation
            if (jbi.getOperation() != null) {
                String operationName = NamingHelper.qNameToString(jbi
                    .getOperation());
                jms.setStringProperty(OPERATION, operationName == null ? ""
                    : operationName);
            }

            // set interface
            if (jbi.getInterfaceName() != null) {
                String interfaceName = NamingHelper.qNameToString(jbi
                    .getInterfaceName());
                jms.setStringProperty(INTERFACE, interfaceName == null ? ""
                    : interfaceName);
            }

            // set endpoint
            if (jbi.getEndpoint() != null) {
                jms.setStringProperty(ENDPOINT, jbi.getEndpoint()
                    .getEndpointName());
                String endpointServiceName = NamingHelper.qNameToString(jbi
                    .getEndpoint().getServiceName());
                jms.setStringProperty(ENDPOINT_SERVICE, endpointServiceName);
            }

            // set status
            jms.setStringProperty(STATUS, jbi.getStatus().toString());

            // set pattern
            jms.setStringProperty(PATTERN, jbi.getPattern().toASCIIString());

            // set Role
            String role = null;
            if (MessageExchange.Role.CONSUMER.equals(jbi.getRole())) {
                role = CONSUMER;
            } else if (MessageExchange.Role.PROVIDER.equals(jbi.getRole())) {
                role = PROVIDER;
            } else {
                throw new JBIException("Role is unknown");
            }
            jms.setStringProperty(ROLE, role);

            // set transacted
            jms.setBooleanProperty(TRANSACTED, jbi.isTransacted());

            // //////////////////////////////////////////////////
            // set properties used for implementation
            // //////////////////////////////////////////////////

            // set terminated
            jms.setBooleanProperty(TERMINATED, jbi.isTerminated());

            // set Msg type

            // Normalize Message
            javax.jbi.messaging.NormalizedMessage nmToSend = null;
            if (MessageExchange.Role.PROVIDER.equals(jbi.getRole())) {
                nmToSend = jbi.getMessage(MessageExchangeImpl.IN_MSG);
            } else {
                nmToSend = jbi.getMessage(MessageExchangeImpl.OUT_MSG);
            }
            if (nmToSend == null) {
                throw new JBIException("normalized message to send is null");
            }
            // set normalized properties
            for (Iterator it = nmToSend.getPropertyNames().iterator(); it
                .hasNext();) {
                String pName = (String) it.next();
                jms.setObjectProperty(pName, nmToSend.getProperty(pName));
            }

            // set content
            StreamResult stream = new StreamResult(new ByteArrayOutputStream());
            Transformer transformer = TransformerFactory.newInstance()
                .newTransformer();
            transformer.transform(nmToSend.getContent(), stream);
            stream.getOutputStream().close();
            jms.setText(stream.getOutputStream().toString());

            // set attachment
            // TODO
        } catch (Exception e) {
            throw new JBIException(e);
        }

        return jms;
    }

    public MessageExchangeImpl jms2jbi(Message jms) throws JBIException {
        // create MessageExchange depending on pattern
        MessageExchangeImpl ms = new MessageExchangeImpl(null);
        try {

            NormalizedMessageImpl nm = new NormalizedMessageImpl();
            StreamSource source = new StreamSource();

            // create normalized message from jms content
            byte[] msgByte = ((TextMessage) jms).getText().getBytes();
            ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
            source.setInputStream(in);
            nm.setContent(source);
            // create normalized message properties from jms properties
            Enumeration e = jms.getPropertyNames();

            while (e.hasMoreElements()) {
                String propertyName = (String) e.nextElement();
                Object propertyValue = jms.getObjectProperty(propertyName);
                nm.setProperty(propertyName, propertyValue);
            }

            // set message exchange information
            ms.setOperation(new QName(jms.getStringProperty(OPERATION)));

            // set exchange properties (interface,service,endpoint)
            QName service = new QName(jms.getStringProperty("service"));
            QName interfaceName = new QName(jms.getStringProperty("interface"));
            // TODO
            ms.setEndpoint(null);
            ms.setService(service);
            ms.setInterfaceName(interfaceName);

            // set Id
            ms.setExchangeId(jms.getStringProperty(ID));

            // set status
            ms.setStatus(ExchangeStatus.valueOf(jms.getStringProperty(STATUS)));

            // set pattern
            ms.setPattern(new URI(jms.getStringProperty(PATTERN)));

            // TODO set role
            if (jms.getStringProperty("role").equals("provider")) {
                ms.setMessage(nm, MessageExchangeImpl.IN_MSG);
                ms.setRole(MessageExchange.Role.PROVIDER);
            } else {
                ms.setMessage(nm, MessageExchangeImpl.OUT_MSG);
                ms.setRole(MessageExchange.Role.CONSUMER);
            }
            // TODO fault, exception
        } catch (Exception e) {
            throw new JBIException(e);
        }
        return ms;
    }

}
