/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemRecoveryTest.java 2:06:20 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.systemstate;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import javax.jbi.JBIException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.lifecycle.Installer;
import org.objectweb.petals.jbi.management.lifecycle.LifeCycleMBean;
import org.objectweb.petals.jbi.management.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.service.DeploymentServiceMBean;
import org.objectweb.petals.jbi.management.service.InstallationServiceMBean;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerService;
import org.objectweb.petals.jbi.management.service.ManagementException;
import org.objectweb.petals.jbi.management.service.PackageHandler;
import org.objectweb.petals.jbi.management.systemstate.mock.MockMBeanServer;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.tools.jbicommon.descriptor.Identification;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceAssembly;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Tests of the SystemRecovery
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class SystemRecoveryTest extends TestCase {

    public void testConstructor() {
        InstallationServiceMBean installationServiceMBean = EasyMock
            .createMock(InstallationServiceMBean.class);
        Logger logger = EasyMock.createMock(Logger.class);
        DeploymentServiceMBean deploymentServiceMBean = EasyMock
            .createMock(DeploymentServiceMBean.class);
        LifeCycleManagerService lifeCycleManagerService = EasyMock
            .createMock(LifeCycleManagerService.class);
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        RepositoryService repositoryService = EasyMock
            .createMock(RepositoryService.class);
        SystemRecovery systemRecovery = new SystemRecovery(
            installationServiceMBean, logger, deploymentServiceMBean,
            lifeCycleManagerService, stateStorage, repositoryService);

        assertEquals(systemRecovery.deploymentService, deploymentServiceMBean);
        assertNotNull(systemRecovery.log);
        assertEquals(systemRecovery.installationService,
            installationServiceMBean);
        assertEquals(systemRecovery.managerService, lifeCycleManagerService);
        assertEquals(systemRecovery.stateStorage, stateStorage);
    }

    public void testRecoverAllComponent() throws SecurityException,
        NoSuchMethodException, ManagementException, PetalsException {
        SystemRecovery systemRecovery = EasyMock.createMock(
            SystemRecovery.class, new Method[] {SystemRecovery.class
                .getDeclaredMethod("recoverComponent", new Class[] {URI.class,
                    JBIDescriptor.class, String.class, String.class})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        PackageHandler packageHandler = EasyMock
            .createMock(PackageHandler.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);

        List<ComponentState> compos = new ArrayList<ComponentState>();
        ComponentState componentState = new ComponentState();
        componentState.setInstallURL("installURL");
        componentState.setArchiveURL("zipURL");
        componentState.setInstallState("installed");
        componentState.setLifecycleState("stopped");
        compos.add(componentState);

        EasyMock.expect(
            packageHandler.processAndGetPackageURI("installURL", true))
            .andReturn(new File(".").toURI());
        EasyMock.expect(stateStorage.retrieveAllComponentStates()).andReturn(
            compos);
        EasyMock.expect(packageHandler.processAndGetPackageURI("zipURL", true))
            .andReturn(new File(".").toURI());
        EasyMock.expect(packageHandler.loadDescriptor(new File(".").toURI()))
            .andReturn(jbiDescriptor);
        systemRecovery.recoverComponent(new File(".").toURI(), jbiDescriptor,
            "installed", "stopped");
        loggingUtil.start();
        loggingUtil.end();

        EasyMock.replay(stateStorage);
        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(packageHandler);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(systemRecovery);

        systemRecovery.log = loggingUtil;
        systemRecovery.packageHandler = packageHandler;
        systemRecovery.stateStorage = stateStorage;

        systemRecovery.recoverAllComponent();
    }

    public void testRecoverAllComponentException() throws SecurityException,
        NoSuchMethodException, ManagementException, PetalsException {
        SystemRecovery systemRecovery = EasyMock.createMock(
            SystemRecovery.class, new Method[] {SystemRecovery.class
                .getDeclaredMethod("recoverComponent", new Class[] {URI.class,
                    JBIDescriptor.class, String.class, String.class})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        PackageHandler packageHandler = EasyMock
            .createMock(PackageHandler.class);

        List<ComponentState> compos = new ArrayList<ComponentState>();
        ComponentState componentState = new ComponentState();
        componentState.setInstallURL("installURL");
        componentState.setArchiveURL("zipURL");
        componentState.setInstallState("installed");
        componentState.setLifecycleState("stopped");
        componentState.setName("compo");
        compos.add(componentState);

        EasyMock.expect(
            packageHandler.processAndGetPackageURI("installURL", true))
            .andThrow(new PetalsException());
        EasyMock.expect(stateStorage.retrieveAllComponentStates()).andReturn(
            compos);
        loggingUtil.start();
        loggingUtil.error("A component can't be recovered : compo");

        EasyMock.replay(stateStorage);
        EasyMock.replay(packageHandler);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(systemRecovery);

        systemRecovery.log = loggingUtil;
        systemRecovery.packageHandler = packageHandler;
        systemRecovery.stateStorage = stateStorage;

        try {
            systemRecovery.recoverAllComponent();
            fail();
        } catch (ManagementException e) {
            // do nothing
        }
    }

    public void testRecoverAllServiceAssembly() throws SecurityException,
        NoSuchMethodException, ManagementException, PetalsException {
        SystemRecovery systemRecovery = EasyMock.createMock(
            SystemRecovery.class, new Method[] {SystemRecovery.class
                .getDeclaredMethod("recoverServiceAssembly", new Class[] {
                    URI.class, JBIDescriptor.class, String.class})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        PackageHandler packageHandler = EasyMock
            .createMock(PackageHandler.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);

        List<ServiceAssemblyState> servList = new ArrayList<ServiceAssemblyState>();
        ServiceAssemblyState serviceAssemblyState = new ServiceAssemblyState();
        serviceAssemblyState.setInstallURL("installURL");
        serviceAssemblyState.setArchiveURL("zipURL");
        serviceAssemblyState.setLifecycleState("stopped");
        servList.add(serviceAssemblyState);

        EasyMock.expect(
            packageHandler.processAndGetPackageURI("installURL", true))
            .andReturn(new File(".").toURI());
        EasyMock.expect(stateStorage.retrieveAllServiceAssemblyStates())
            .andReturn(servList);
        EasyMock.expect(packageHandler.processAndGetPackageURI("zipURL", true))
            .andReturn(new File(".").toURI());
        EasyMock.expect(packageHandler.loadDescriptor(new File(".").toURI()))
            .andReturn(jbiDescriptor);
        systemRecovery.recoverServiceAssembly(new File(".").toURI(),
            jbiDescriptor, "stopped");
        loggingUtil.start();
        loggingUtil.end();

        EasyMock.replay(stateStorage);
        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(packageHandler);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(systemRecovery);

        systemRecovery.log = loggingUtil;
        systemRecovery.packageHandler = packageHandler;
        systemRecovery.stateStorage = stateStorage;

        systemRecovery.recoverAllServiceAssembly();
    }

    public void testRecoverAllServiceAssemblyException()
        throws SecurityException, NoSuchMethodException, ManagementException,
        PetalsException {
        SystemRecovery systemRecovery = EasyMock.createMock(
            SystemRecovery.class, new Method[] {SystemRecovery.class
                .getDeclaredMethod("recoverServiceAssembly", new Class[] {
                    URI.class, JBIDescriptor.class, String.class})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        PackageHandler packageHandler = EasyMock
            .createMock(PackageHandler.class);

        List<ServiceAssemblyState> servList = new ArrayList<ServiceAssemblyState>();
        ServiceAssemblyState serviceAssemblyState = new ServiceAssemblyState();
        serviceAssemblyState.setInstallURL("installURL");
        serviceAssemblyState.setArchiveURL("zipURL");
        serviceAssemblyState.setLifecycleState("stopped");
        serviceAssemblyState.setName("name");
        servList.add(serviceAssemblyState);

        EasyMock.expect(
            packageHandler.processAndGetPackageURI("installURL", true))
            .andThrow(new PetalsException());
        EasyMock.expect(stateStorage.retrieveAllServiceAssemblyStates())
            .andReturn(servList);
        loggingUtil.start();
        loggingUtil.error("A service assembly can't be recovered : name");

        EasyMock.replay(stateStorage);
        EasyMock.replay(packageHandler);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(systemRecovery);

        systemRecovery.log = loggingUtil;
        systemRecovery.packageHandler = packageHandler;
        systemRecovery.stateStorage = stateStorage;

        try {
            systemRecovery.recoverAllServiceAssembly();
            fail();
        } catch (ManagementException e) {
            // do nothing
        }
    }

    public void testRecoverAllSharedLibrary() throws SecurityException,
        NoSuchMethodException, ManagementException, PetalsException {
        SystemRecovery systemRecovery = EasyMock.createMock(
            SystemRecovery.class, new Method[] {SystemRecovery.class
                .getDeclaredMethod("recoverSharedLib", new Class[] {URI.class,
                    JBIDescriptor.class})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        PackageHandler packageHandler = EasyMock
            .createMock(PackageHandler.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);

        List<SharedLibraryState> servList = new ArrayList<SharedLibraryState>();
        SharedLibraryState sharedLibraryState = new SharedLibraryState();
        sharedLibraryState.setInstallURL("installURL");
        sharedLibraryState.setArchiveURL("zipURL");
        servList.add(sharedLibraryState);

        EasyMock.expect(
            packageHandler.processAndGetPackageURI("installURL", true))
            .andReturn(new File(".").toURI());
        EasyMock.expect(stateStorage.retrieveAllSharedLibraryStates())
            .andReturn(servList);
        EasyMock.expect(packageHandler.processAndGetPackageURI("zipURL", true))
            .andReturn(new File(".").toURI());
        EasyMock.expect(packageHandler.loadDescriptor(new File(".").toURI()))
            .andReturn(jbiDescriptor);
        systemRecovery.recoverSharedLib(new File(".").toURI(), jbiDescriptor);
        loggingUtil.start();
        loggingUtil.end();

        EasyMock.replay(stateStorage);
        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(packageHandler);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(systemRecovery);

        systemRecovery.log = loggingUtil;
        systemRecovery.packageHandler = packageHandler;
        systemRecovery.stateStorage = stateStorage;

        systemRecovery.recoverAllSharedLibrary();
    }

    public void testRecoverAllSharedLibraryException()
        throws SecurityException, NoSuchMethodException, ManagementException,
        PetalsException {
        SystemRecovery systemRecovery = EasyMock.createMock(
            SystemRecovery.class, new Method[] {SystemRecovery.class
                .getDeclaredMethod("recoverSharedLib", new Class[] {URI.class,
                    JBIDescriptor.class})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        PackageHandler packageHandler = EasyMock
            .createMock(PackageHandler.class);

        List<SharedLibraryState> servList = new ArrayList<SharedLibraryState>();
        SharedLibraryState sharedLibraryState = new SharedLibraryState();
        sharedLibraryState.setInstallURL("installURL");
        sharedLibraryState.setArchiveURL("zipURL");
        sharedLibraryState.setName("name");
        servList.add(sharedLibraryState);

        EasyMock.expect(
            packageHandler.processAndGetPackageURI("installURL", true))
            .andThrow(new PetalsException());
        EasyMock.expect(stateStorage.retrieveAllSharedLibraryStates())
            .andReturn(servList);
        loggingUtil.start();
        loggingUtil.error("A Shared lib can't be recovered : name");

        EasyMock.replay(stateStorage);
        EasyMock.replay(packageHandler);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(systemRecovery);

        systemRecovery.log = loggingUtil;
        systemRecovery.packageHandler = packageHandler;
        systemRecovery.stateStorage = stateStorage;

        try {
            systemRecovery.recoverAllSharedLibrary();
            fail();
        } catch (ManagementException e) {
            // do nothing
        }
    }

    public void testRecoverComponentStopped() throws ManagementException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);
        ComponentDescription componentDescription = EasyMock
            .createMock(ComponentDescription.class);
        Identification identification = EasyMock
            .createMock(Identification.class);
        LifeCycleManagerService lifeCycleManagerService = EasyMock
            .createMock(LifeCycleManagerService.class);
        MockMBeanServer mBeanServer = new MockMBeanServer();
        InstallationServiceMBean installationServiceMBean = EasyMock
            .createMock(InstallationServiceMBean.class);

        loggingUtil.start();
        EasyMock.expect(jbiDescriptor.getComponent()).andReturn(
            componentDescription);
        EasyMock.expect(componentDescription.getIdentification()).andReturn(
            identification);
        EasyMock.expect(identification.getName()).andReturn("compo");
        loggingUtil.end();
        EasyMock.expect(lifeCycleManagerService.getMBeanServer()).andReturn(
            mBeanServer);
        EasyMock.expect(
            installationServiceMBean.loadNewInstaller(new File(".").toURI(),
                jbiDescriptor)).andReturn(null);

        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(installationServiceMBean);
        EasyMock.replay(componentDescription);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(lifeCycleManagerService);
        EasyMock.replay(identification);

        SystemRecovery systemRecovery = new SystemRecovery(
            installationServiceMBean, EasyMock.createMock(Logger.class),
            EasyMock.createMock(DeploymentServiceMBean.class),
            lifeCycleManagerService, null, null);
        systemRecovery.log = loggingUtil;

        systemRecovery.recoverComponent(new File(".").toURI(), jbiDescriptor,
            Installer.INSTALLED, LifeCycleMBean.STOPPED);
    }

    public void testRecoverComponentStarted() throws ManagementException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);
        ComponentDescription componentDescription = EasyMock
            .createMock(ComponentDescription.class);
        Identification identification = EasyMock
            .createMock(Identification.class);
        LifeCycleManagerService lifeCycleManagerService = EasyMock
            .createMock(LifeCycleManagerService.class);
        MockMBeanServer mBeanServer = new MockMBeanServer();
        InstallationServiceMBean installationServiceMBean = EasyMock
            .createMock(InstallationServiceMBean.class);

        loggingUtil.start();
        EasyMock.expect(jbiDescriptor.getComponent()).andReturn(
            componentDescription);
        EasyMock.expect(componentDescription.getIdentification()).andReturn(
            identification);
        EasyMock.expect(identification.getName()).andReturn("compo");
        loggingUtil.end();
        EasyMock.expect(lifeCycleManagerService.getMBeanServer()).andReturn(
            mBeanServer);
        EasyMock.expect(
            installationServiceMBean.loadNewInstaller(new File(".").toURI(),
                jbiDescriptor)).andReturn(null);

        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(installationServiceMBean);
        EasyMock.replay(componentDescription);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(lifeCycleManagerService);
        EasyMock.replay(identification);

        SystemRecovery systemRecovery = new SystemRecovery(
            installationServiceMBean, EasyMock.createMock(Logger.class),
            EasyMock.createMock(DeploymentServiceMBean.class),
            lifeCycleManagerService, null, null);
        systemRecovery.log = loggingUtil;

        systemRecovery.recoverComponent(new File(".").toURI(), jbiDescriptor,
            Installer.INSTALLED, LifeCycleMBean.STARTED);
    }

    public void testRecoverComponentExceptionThrowInstanceNotFoundException()
        throws ManagementException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);
        ComponentDescription componentDescription = EasyMock
            .createMock(ComponentDescription.class);
        Identification identification = EasyMock
            .createMock(Identification.class);
        LifeCycleManagerService lifeCycleManagerService = EasyMock
            .createMock(LifeCycleManagerService.class);
        MockMBeanServer mBeanServer = new MockMBeanServer();
        InstallationServiceMBean installationServiceMBean = EasyMock
            .createMock(InstallationServiceMBean.class);

        loggingUtil.start();
        EasyMock.expect(jbiDescriptor.getComponent()).andReturn(
            componentDescription);
        EasyMock.expect(componentDescription.getIdentification()).andReturn(
            identification);
        EasyMock.expect(identification.getName()).andReturn("compo");
        EasyMock.expect(lifeCycleManagerService.getMBeanServer()).andReturn(
            mBeanServer);
        EasyMock.expect(
            installationServiceMBean.loadNewInstaller(new File(".").toURI(),
                jbiDescriptor)).andReturn(null);
        loggingUtil.error("Component can't be recovered : compo");

        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(installationServiceMBean);
        EasyMock.replay(componentDescription);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(lifeCycleManagerService);
        EasyMock.replay(identification);

        SystemRecovery systemRecovery = new SystemRecovery(
            installationServiceMBean, EasyMock.createMock(Logger.class),
            EasyMock.createMock(DeploymentServiceMBean.class),
            lifeCycleManagerService, null, null);
        systemRecovery.log = loggingUtil;

        mBeanServer.setThrowInstanceNotFoundException(true);

        try {
            systemRecovery.recoverComponent(new File(".").toURI(),
                jbiDescriptor, Installer.INSTALLED, LifeCycleMBean.STOPPED);
            fail();
        } catch (ManagementException e) {
            // do nothing
        }
    }

    public void testRecoverComponentExceptionThrowMBeanException()
        throws ManagementException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);
        ComponentDescription componentDescription = EasyMock
            .createMock(ComponentDescription.class);
        Identification identification = EasyMock
            .createMock(Identification.class);
        LifeCycleManagerService lifeCycleManagerService = EasyMock
            .createMock(LifeCycleManagerService.class);
        MockMBeanServer mBeanServer = new MockMBeanServer();
        InstallationServiceMBean installationServiceMBean = EasyMock
            .createMock(InstallationServiceMBean.class);

        loggingUtil.start();
        EasyMock.expect(jbiDescriptor.getComponent()).andReturn(
            componentDescription);
        EasyMock.expect(componentDescription.getIdentification()).andReturn(
            identification);
        EasyMock.expect(identification.getName()).andReturn("compo");
        EasyMock.expect(lifeCycleManagerService.getMBeanServer()).andReturn(
            mBeanServer);
        EasyMock.expect(
            installationServiceMBean.loadNewInstaller(new File(".").toURI(),
                jbiDescriptor)).andReturn(null);
        loggingUtil.error("Component can't be recovered : compo");

        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(installationServiceMBean);
        EasyMock.replay(componentDescription);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(lifeCycleManagerService);
        EasyMock.replay(identification);

        SystemRecovery systemRecovery = new SystemRecovery(
            installationServiceMBean, EasyMock.createMock(Logger.class),
            EasyMock.createMock(DeploymentServiceMBean.class),
            lifeCycleManagerService, null, null);
        systemRecovery.log = loggingUtil;

        mBeanServer.setThrowMBeanException(true);

        try {
            systemRecovery.recoverComponent(new File(".").toURI(),
                jbiDescriptor, Installer.INSTALLED, LifeCycleMBean.STOPPED);
            fail();
        } catch (ManagementException e) {
            // do nothing
        }
    }

    public void testRecoverComponentExceptionThrowReflectionException()
        throws ManagementException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);
        ComponentDescription componentDescription = EasyMock
            .createMock(ComponentDescription.class);
        Identification identification = EasyMock
            .createMock(Identification.class);
        LifeCycleManagerService lifeCycleManagerService = EasyMock
            .createMock(LifeCycleManagerService.class);
        MockMBeanServer mBeanServer = new MockMBeanServer();
        InstallationServiceMBean installationServiceMBean = EasyMock
            .createMock(InstallationServiceMBean.class);

        loggingUtil.start();
        EasyMock.expect(jbiDescriptor.getComponent()).andReturn(
            componentDescription);
        EasyMock.expect(componentDescription.getIdentification()).andReturn(
            identification);
        EasyMock.expect(identification.getName()).andReturn("compo");
        EasyMock.expect(lifeCycleManagerService.getMBeanServer()).andReturn(
            mBeanServer);
        EasyMock.expect(
            installationServiceMBean.loadNewInstaller(new File(".").toURI(),
                jbiDescriptor)).andReturn(null);
        loggingUtil.error("Component can't be recovered : compo");

        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(installationServiceMBean);
        EasyMock.replay(componentDescription);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(lifeCycleManagerService);
        EasyMock.replay(identification);

        SystemRecovery systemRecovery = new SystemRecovery(
            installationServiceMBean, EasyMock.createMock(Logger.class),
            EasyMock.createMock(DeploymentServiceMBean.class),
            lifeCycleManagerService, null, null);
        systemRecovery.log = loggingUtil;

        mBeanServer.setThrowReflectionException(true);

        try {
            systemRecovery.recoverComponent(new File(".").toURI(),
                jbiDescriptor, Installer.INSTALLED, LifeCycleMBean.STOPPED);
            fail();
        } catch (ManagementException e) {
            // do nothing
        }
    }

    public void testRecoverServiceAssemblyStopped() throws PetalsException,
        JBIException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);
        ServiceAssembly serviceAssembly = EasyMock
            .createMock(ServiceAssembly.class);
        Identification identification = EasyMock
            .createMock(Identification.class);
        LifeCycleManagerService lifeCycleManagerService = EasyMock
            .createMock(LifeCycleManagerService.class);
        MockMBeanServer mBeanServer = new MockMBeanServer();
        InstallationServiceMBean installationServiceMBean = EasyMock
            .createMock(InstallationServiceMBean.class);
        DeploymentServiceMBean deploymentServiceMBean = EasyMock
            .createMock(DeploymentServiceMBean.class);
        ServiceAssemblyLifeCycle serviceAssemblyLifeCycle = EasyMock
            .createMock(ServiceAssemblyLifeCycle.class);

        loggingUtil.start();
        EasyMock.expect(jbiDescriptor.getServiceAssembly()).andReturn(
            serviceAssembly);
        EasyMock.expect(serviceAssembly.getIdentification()).andReturn(
            identification);
        EasyMock.expect(identification.getName()).andReturn("sa");
        EasyMock.expect(lifeCycleManagerService.getMBeanServer()).andReturn(
            mBeanServer);
        EasyMock.expect(
            installationServiceMBean.loadNewInstaller(new File(".").toURI(),
                jbiDescriptor)).andReturn(null);
        EasyMock.expect(lifeCycleManagerService.getServiceAssemblyByName("sa"))
            .andReturn(serviceAssemblyLifeCycle);
        EasyMock
            .expect(
                deploymentServiceMBean.deploy(new File(".").toURI(),
                    jbiDescriptor)).andReturn("success");
        serviceAssemblyLifeCycle.start();
        serviceAssemblyLifeCycle.stop();
        loggingUtil.end();

        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(serviceAssemblyLifeCycle);
        EasyMock.replay(installationServiceMBean);
        EasyMock.replay(serviceAssembly);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(lifeCycleManagerService);
        EasyMock.replay(identification);
        EasyMock.replay(deploymentServiceMBean);

        SystemRecovery systemRecovery = new SystemRecovery(
            installationServiceMBean, EasyMock.createMock(Logger.class),
            deploymentServiceMBean, lifeCycleManagerService, null, null);
        systemRecovery.log = loggingUtil;

        systemRecovery.recoverServiceAssembly(new File(".").toURI(),
            jbiDescriptor, LifeCycleMBean.STOPPED);
    }

    public void testRecoverServiceAssemblyStarted() throws PetalsException,
        JBIException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);
        ServiceAssembly serviceAssembly = EasyMock
            .createMock(ServiceAssembly.class);
        Identification identification = EasyMock
            .createMock(Identification.class);
        LifeCycleManagerService lifeCycleManagerService = EasyMock
            .createMock(LifeCycleManagerService.class);
        MockMBeanServer mBeanServer = new MockMBeanServer();
        InstallationServiceMBean installationServiceMBean = EasyMock
            .createMock(InstallationServiceMBean.class);
        DeploymentServiceMBean deploymentServiceMBean = EasyMock
            .createMock(DeploymentServiceMBean.class);
        ServiceAssemblyLifeCycle serviceAssemblyLifeCycle = EasyMock
            .createMock(ServiceAssemblyLifeCycle.class);

        loggingUtil.start();
        EasyMock.expect(jbiDescriptor.getServiceAssembly()).andReturn(
            serviceAssembly);
        EasyMock.expect(serviceAssembly.getIdentification()).andReturn(
            identification);
        EasyMock.expect(identification.getName()).andReturn("sa");
        EasyMock.expect(lifeCycleManagerService.getMBeanServer()).andReturn(
            mBeanServer);
        EasyMock.expect(
            installationServiceMBean.loadNewInstaller(new File(".").toURI(),
                jbiDescriptor)).andReturn(null);
        EasyMock.expect(lifeCycleManagerService.getServiceAssemblyByName("sa"))
            .andReturn(serviceAssemblyLifeCycle);
        EasyMock
            .expect(
                deploymentServiceMBean.deploy(new File(".").toURI(),
                    jbiDescriptor)).andReturn("success");
        serviceAssemblyLifeCycle.start();
        serviceAssemblyLifeCycle.stop();
        loggingUtil.end();

        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(serviceAssemblyLifeCycle);
        EasyMock.replay(installationServiceMBean);
        EasyMock.replay(serviceAssembly);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(lifeCycleManagerService);
        EasyMock.replay(identification);
        EasyMock.replay(deploymentServiceMBean);

        SystemRecovery systemRecovery = new SystemRecovery(
            installationServiceMBean, EasyMock.createMock(Logger.class),
            deploymentServiceMBean, lifeCycleManagerService, null, null);
        systemRecovery.log = loggingUtil;

        systemRecovery.recoverServiceAssembly(new File(".").toURI(),
            jbiDescriptor, LifeCycleMBean.STARTED);
    }

    public void testRecoverServiceAssemblyPetalsException()
        throws PetalsException, JBIException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);
        ServiceAssembly serviceAssembly = EasyMock
            .createMock(ServiceAssembly.class);
        Identification identification = EasyMock
            .createMock(Identification.class);
        LifeCycleManagerService lifeCycleManagerService = EasyMock
            .createMock(LifeCycleManagerService.class);
        MockMBeanServer mBeanServer = new MockMBeanServer();
        InstallationServiceMBean installationServiceMBean = EasyMock
            .createMock(InstallationServiceMBean.class);
        DeploymentServiceMBean deploymentServiceMBean = EasyMock
            .createMock(DeploymentServiceMBean.class);
        ServiceAssemblyLifeCycle serviceAssemblyLifeCycle = EasyMock
            .createMock(ServiceAssemblyLifeCycle.class);

        loggingUtil.start();
        EasyMock.expect(jbiDescriptor.getServiceAssembly()).andReturn(
            serviceAssembly);
        EasyMock.expect(serviceAssembly.getIdentification()).andReturn(
            identification);
        EasyMock.expect(identification.getName()).andReturn("sa");
        EasyMock.expect(lifeCycleManagerService.getMBeanServer()).andReturn(
            mBeanServer);
        EasyMock.expect(
            installationServiceMBean.loadNewInstaller(new File(".").toURI(),
                jbiDescriptor)).andReturn(null);
        EasyMock.expect(lifeCycleManagerService.getServiceAssemblyByName("sa"))
            .andReturn(serviceAssemblyLifeCycle);
        EasyMock
            .expect(
                deploymentServiceMBean.deploy(new File(".").toURI(),
                    jbiDescriptor)).andThrow(new PetalsException());
        serviceAssemblyLifeCycle.start();
        serviceAssemblyLifeCycle.stop();
        loggingUtil.error("ServiceAssembly can't be recovered : sa");

        EasyMock.replay(jbiDescriptor);
        EasyMock.replay(serviceAssemblyLifeCycle);
        EasyMock.replay(installationServiceMBean);
        EasyMock.replay(serviceAssembly);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(lifeCycleManagerService);
        EasyMock.replay(identification);
        EasyMock.replay(deploymentServiceMBean);

        SystemRecovery systemRecovery = new SystemRecovery(
            installationServiceMBean, EasyMock.createMock(Logger.class),
            deploymentServiceMBean, lifeCycleManagerService, null, null);
        systemRecovery.log = loggingUtil;

        try {
            systemRecovery.recoverServiceAssembly(new File(".").toURI(),
                jbiDescriptor, LifeCycleMBean.STARTED);
            fail();
        } catch (ManagementException e) {
            // do nothing
        }
    }

    public void testRecoverSharedLib() throws ManagementException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        InstallationServiceMBean installationServiceMBean = EasyMock
            .createMock(InstallationServiceMBean.class);
        JBIDescriptor jbiDescriptor = EasyMock.createMock(JBIDescriptor.class);

        loggingUtil.start();
        EasyMock.expect(
            installationServiceMBean.installSharedLibrary(
                new File(".").toURI(), jbiDescriptor)).andReturn("success");
        loggingUtil.end();

        EasyMock.replay(installationServiceMBean);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(jbiDescriptor);

        SystemRecovery systemRecovery = new SystemRecovery(
            installationServiceMBean, EasyMock.createMock(Logger.class),
            EasyMock.createMock(DeploymentServiceMBean.class), EasyMock
                .createMock(LifeCycleManagerService.class), null, null);
        systemRecovery.log = loggingUtil;

        systemRecovery.recoverSharedLib(new File(".").toURI(), jbiDescriptor);
    }

}
