/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MessageSerializerTest.java 9:38:24 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.transport;

import java.util.Vector;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jms.JMSException;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.joram.client.jms.QueueSession;
import org.objectweb.joram.client.jms.TextMessage;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.messaging.NormalizedMessageImpl;
import org.objectweb.petals.jbi.registry.ConsumerEndpoint;
import org.objectweb.petals.jbi.registry.InternalEndpoint;
import org.objectweb.petals.util.SourceHelper;

/**
 * Test of the MessageSerializer
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class MessageSerializerTest extends TestCase {

    public void testJbi2jmsJBIException() throws JBIException, JMSException {
        MessageSerializer messageSerializer = new MessageSerializer();
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));
        messageExchangeImpl.setService(new QName("service"));
        messageExchangeImpl.setOperation(new QName("operation"));
        messageExchangeImpl.setInterfaceName(new QName("interface"));
        messageExchangeImpl.setEndpoint(new InternalEndpoint(new QName(
            "service"), "endpoint", "compo", "0", null));
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_ONLY_PATTERN);
        messageExchangeImpl.setRole(Role.CONSUMER);

        QueueSession queueSession = EasyMock.createMock(QueueSession.class);
        TextMessage textMessage = EasyMock.createMock(TextMessage.class);

        EasyMock.expect(queueSession.createTextMessage())
            .andReturn(textMessage).anyTimes();

        textMessage.setStringProperty("id", null);
        textMessage.setStringProperty("service", "service");
        textMessage.setStringProperty("operation", "operation");
        textMessage.setStringProperty("interface", "interface");
        textMessage.setStringProperty("endpoint-name", "endpoint");
        textMessage.setStringProperty("endpoint-servicename", "service");
        textMessage.setStringProperty("status", "Active");
        textMessage.setStringProperty("pattern",
            MessageExchangeImpl.IN_ONLY_PATTERN.toASCIIString());
        textMessage.setStringProperty("role", "consumer");
        textMessage.setBooleanProperty("transacted", false);
        textMessage.setBooleanProperty("terminated", false);
        queueSession.close();

        EasyMock.replay(textMessage);
        EasyMock.replay(queueSession);

        try {
            messageSerializer.jbi2jms(messageExchangeImpl, queueSession);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testJbi2jms() throws JBIException, JMSException {
        MessageSerializer messageSerializer = new MessageSerializer();
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));
        messageExchangeImpl.setService(new QName("service"));
        messageExchangeImpl.setOperation(new QName("operation"));
        messageExchangeImpl.setInterfaceName(new QName("interface"));
        messageExchangeImpl.setEndpoint(new InternalEndpoint(new QName(
            "service"), "endpoint", "compo", "0", null));
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_OUT_PATTERN);
        NormalizedMessageImpl normalizedMessageImpl = new NormalizedMessageImpl();
        normalizedMessageImpl.setContent(SourceHelper.createSource("<test />"));
        messageExchangeImpl.setMessage(normalizedMessageImpl, "in");
        messageExchangeImpl.setRole(Role.PROVIDER);

        QueueSession queueSession = EasyMock.createMock(QueueSession.class);
        TextMessage textMessage = EasyMock.createMock(TextMessage.class);

        EasyMock.expect(queueSession.createTextMessage())
            .andReturn(textMessage).anyTimes();

        textMessage.setStringProperty("id", null);
        textMessage.setStringProperty("service", "service");
        textMessage.setStringProperty("operation", "operation");
        textMessage.setStringProperty("interface", "interface");
        textMessage.setStringProperty("endpoint-name", "endpoint");
        textMessage.setStringProperty("endpoint-servicename", "service");
        textMessage.setStringProperty("status", "Active");
        textMessage.setStringProperty("pattern",
            MessageExchangeImpl.IN_OUT_PATTERN.toASCIIString());
        textMessage.setStringProperty("role", "provider");
        textMessage.setBooleanProperty("transacted", false);
        textMessage.setBooleanProperty("terminated", false);
        textMessage
            .setText("<?xml version=\"1.0\" encoding=\"UTF-8\"?><test/>");
        queueSession.close();

        EasyMock.replay(textMessage);
        EasyMock.replay(queueSession);

        assertEquals(messageSerializer.jbi2jms(messageExchangeImpl,
            queueSession), textMessage);
    }

    public void testJms2jbi() throws JBIException, JMSException {
        MessageSerializer messageSerializer = new MessageSerializer();
        TextMessage textMessage = EasyMock.createMock(TextMessage.class);

        EasyMock.expect(textMessage.getText()).andReturn("<test />").anyTimes();
        EasyMock.expect(textMessage.getPropertyNames()).andReturn(
            new Vector<String>().elements()).anyTimes();
        EasyMock.expect(textMessage.getStringProperty("operation")).andReturn(
            "operation").anyTimes();
        EasyMock.expect(textMessage.getStringProperty("service")).andReturn(
            "service").anyTimes();
        EasyMock.expect(textMessage.getStringProperty("interface")).andReturn(
            "interface").anyTimes();
        EasyMock.expect(textMessage.getStringProperty("id")).andReturn("1")
            .anyTimes();
        EasyMock.expect(textMessage.getStringProperty("status")).andReturn(
            "Active").anyTimes();
        EasyMock.expect(textMessage.getStringProperty("pattern")).andReturn(
            MessageExchangeImpl.IN_OUT_PATTERN.toASCIIString()).anyTimes();
        EasyMock.expect(textMessage.getStringProperty("role")).andReturn(
            "provider").anyTimes();

        EasyMock.replay(textMessage);

        MessageExchangeImpl ms = messageSerializer.jms2jbi(textMessage);
        assertEquals(ms.getRole(), Role.PROVIDER);
    }
}
