/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RegistryServerTest.java 4:12:33 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.registry;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.kernel.registry.msg.response.ExceptionResponse;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Tests of the RegistryServer
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class RegistryServerTest extends TestCase {

    public void testBindFcUnbindFc() throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        RegistryServer server = new RegistryServer();
        server.bindFc(InterfaceNamesHelper.LOGGER_NAME, EasyMock
            .createMock(Logger.class));
        assertNotNull(server.logger);
        server.unbindFc(InterfaceNamesHelper.LOGGER_NAME);
        assertNull(server.logger);
    }

    public void testBindFcException() {
        RegistryServer server = new RegistryServer();
        try {
            server.bindFc("test", "value");
            fail();
        } catch (Exception e) {
            // Do nothing
        }
    }

    public void testUnBindFcException() {
        RegistryServer server = new RegistryServer();
        try {
            server.unbindFc("test");
            fail();
        } catch (Exception e) {
            // Do nothing
        }
    }

    public void testGetFcState() {
        RegistryServer server = new RegistryServer();
        assertEquals(server.getFcState(), LifeCycleController.STOPPED);
    }

    public void testListFc() {
        RegistryServer server = new RegistryServer();
        assertEquals(server.listFc().length, 0);
    }

    public void testLookupFc() throws NoSuchInterfaceException {
        RegistryServer server = new RegistryServer();
        Logger logger = EasyMock.createMock(Logger.class);
        server.logger = logger;
        assertEquals(logger, server.lookupFc(InterfaceNamesHelper.LOGGER_NAME));
    }

    public void testLookupFcException() {
        RegistryServer server = new RegistryServer();
        try {
            server.lookupFc("test");
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testProcessResponseException() {
        RegistryServer server = new RegistryServer();
        ExceptionResponse exceptionResponse = new ExceptionResponse(
            new RuntimeException(), 0, 0);
        assertNull(server.processResponse(exceptionResponse));
    }

    public void testSetData() {
        RegistryServer server = new RegistryServer();
        Map<String, Map<String, Object>> map = new HashMap<String, Map<String, Object>>();
        server.setData(map);
        assertEquals(server.getData(), map);
    }

    public void testSetMaster() {
        RegistryServer server = new RegistryServer();
        server.setMaster(1);
        assertEquals(server.getMaster(), 1);
    }

    public void testStartFc() throws SecurityException, NoSuchMethodException,
        IllegalLifeCycleException {
        RegistryServer server = EasyMock.createMock(RegistryServer.class,
            new Method[] {RegistryServer.class.getDeclaredMethod("startServer",
                new Class[0])});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);

        loggingUtil.call();
        server.startServer();

        EasyMock.replay(loggingUtil);
        EasyMock.replay(server);

        server.log = loggingUtil;

        server.startFc();
        assertEquals(server.fcState, LifeCycleController.STARTED);
    }

    public void testStopFc() throws SecurityException, NoSuchMethodException,
        IllegalLifeCycleException {
        RegistryServer server = EasyMock.createMock(RegistryServer.class,
            new Method[] {RegistryServer.class.getDeclaredMethod("stopServer",
                new Class[0])});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);

        loggingUtil.start();
        server.stopServer();
        loggingUtil.end();

        EasyMock.replay(loggingUtil);
        EasyMock.replay(server);

        server.log = loggingUtil;

        server.stopFc();
        assertEquals(server.fcState, LifeCycleController.STOPPED);
    }

}
