/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RepositoryImplTest.java 10:49:02 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.repository;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import junit.framework.TestCase;

/**
 * Test the RepositoryImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class RepositoryImplTest extends TestCase {

    private String baseDir;

    public void setUp() throws IOException {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        System.setProperty("petals.home", baseDir + "target" + File.separator);
        new File(baseDir + "target" + File.separator + "lib").mkdir();
        new File(baseDir + "target" + File.separator + "lib" + File.separator
            + "petals-1.0-SNAPSHOT.jar").mkdir();
    }

    public void testAddPackage() throws Exception {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        repositoryImpl.addPackage("0001", new File(baseDir + "src"
            + File.separator + "test-data" + File.separator + "test.zip")
            .toURI());
    }

    public void testAddPackage1() throws Exception {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        try {
            repositoryImpl.addPackage("0001", new File(baseDir + "src"
                + File.separator + "test-data" + File.separator + "foo.zip")
                .toURI());
            fail();
        } catch (Exception e) {

        }
    }

    public void testContainsPackage() throws Exception {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        repositoryImpl.addPackage("0001", new File(baseDir + "src"
            + File.separator + "test-data" + File.separator + "test.zip")
            .toURI());
        assertTrue(repositoryImpl.containsPackage("0001"));
    }

    public void testExplodeSUIntoSAInstallFoldere() throws Exception {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        repositoryImpl.explodeSUIntoSAInstallFolder("0001", new File(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "test.zip").toURI(), "0002");
    }

    public void testExplodeSUIntoSAInstallFoldere1() throws Exception {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        new File(baseDir + "repository" + File.separator).delete();
        repositoryImpl.explodeSUIntoSAInstallFolder("0001", new File(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "test.zip").toURI(), "0002");
    }

    public void testGetComponentInstallRoot() throws Exception {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        File dir = repositoryImpl.getComponentInstallRoot("foo");
        assertEquals(dir, new File(baseDir + "target" + File.separator
            + "repository" + File.separator + "foo" + File.separator
            + "install"));
    }

    public void testGetComponentRoot() throws Exception {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        File dir = repositoryImpl.getComponentRoot("foo");
        assertEquals(dir, new File(baseDir + "target" + File.separator
            + "repository" + File.separator + "foo"));
    }

    public void testGetComponentWorkRoot() throws Exception {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        File dir = repositoryImpl.getComponentWorkRoot("foo");
        assertEquals(dir, new File(baseDir + "target" + File.separator
            + "repository" + File.separator + "foo" + File.separator + "work"));
    }

    public void testBindFcLookupFcUnbindFc() throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        Logger logger = EasyMock.createMock(Logger.class);

        repositoryImpl.bindFc(InterfaceNamesHelper.LOGGER_NAME, logger);
        assertEquals(logger, repositoryImpl
            .lookupFc(InterfaceNamesHelper.LOGGER_NAME));
        repositoryImpl.unbindFc(InterfaceNamesHelper.LOGGER_NAME);
        assertNull(repositoryImpl.logger);
        
        repositoryImpl.bindFc(InterfaceNamesHelper.MONOLOG_FACTORY_NAME, null);
        repositoryImpl.unbindFc(InterfaceNamesHelper.MONOLOG_FACTORY_NAME);

        try {
            repositoryImpl.bindFc("foo", "bar");
            fail();
        } catch (Exception e) {
            // do nothing
        }

        try {
            repositoryImpl.unbindFc("foo");
            fail();
        } catch (Exception e) {
            // do nothing
        }

        try {
            repositoryImpl.lookupFc("foo");
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetFcState() {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        repositoryImpl.fcState = LifeCycleController.STARTED;
        assertEquals(repositoryImpl.getFcState(), LifeCycleController.STARTED);
        repositoryImpl.fcState = LifeCycleController.STOPPED;
        assertEquals(repositoryImpl.getFcState(), LifeCycleController.STOPPED);
    }

    public void testListFc() {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        assertEquals(repositoryImpl.listFc().length, 0);
    }

    public void testStartFc() throws SecurityException, NoSuchMethodException,
        IllegalLifeCycleException {
        RepositoryImpl repositoryImpl = EasyMock.createMock(
            RepositoryImpl.class, new Method[] {RepositoryImpl.class
                .getDeclaredMethod("init", new Class[] {})});

        repositoryImpl.init();

        EasyMock.replay(repositoryImpl);

        repositoryImpl.startFc();
        assertEquals(repositoryImpl.fcState, LifeCycleController.STARTED);
    }

    public void testStopFc() throws SecurityException, NoSuchMethodException,
        IllegalLifeCycleException {
        RepositoryImpl repositoryImpl = EasyMock.createMock(
            RepositoryImpl.class, new Method[] {RepositoryImpl.class
                .getDeclaredMethod("shutdown", new Class[] {})});

        repositoryImpl.shutdown();

        EasyMock.replay(repositoryImpl);

        repositoryImpl.stopFc();
        assertEquals(repositoryImpl.fcState, LifeCycleController.STOPPED);
    }

    public void testInit() {
        RepositoryImpl repositoryImpl = new RepositoryImpl();
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_DEBUG, "[START] RepositoryImpl.init()");
        logger.log(BasicLevel.LEVEL_DEBUG, "[ END ] RepositoryImpl.init()");
        
        EasyMock.replay(logger);
        repositoryImpl.logger = logger;
        repositoryImpl.init();
        assertNotNull(repositoryImpl.log);
    }
}
